@@ -26,12 +26,18 @@
 # SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 """
-PuLP: An LP modeler in Python
-
 PuLP is an LP modeler written in python. PuLP can generate MPS or LP files
-and call GLPK, COIN CLP/CBC, CPLEX and XPRESS to solve linear problems.
+and call GLPK[1], COIN CLP/CBC[2], CPLEX[3], and GUROBI[4] to solve linear
+problems.
 
-A comprehensive wiki can be found at http://pulp-or.googlecode.com/
+See the examples directory for examples.
+
+PuLP requires Python >= 2.5.
+
+The examples require at least a solver in your PATH or a shared library file.
+
+Documentation is found on https://www.coin-or.org/PuLP/.
+A comprehensive wiki can be found at https://www.coin-or.org/PuLP/
 
 Use LpVariable() to create new variables. To create a variable 0 <= x <= 3
 >>> x = LpVariable("x", 0, 3)
@@ -74,8 +80,17 @@ Exported Functions:
     - lpSum() -- given a list of the form [a1*x1, a2x2, ..., anxn] will construct 
       a linear expression to be used as a constraint or variable
     - lpDot() --given two lists of the form [a1, a2, ..., an] and 
-      [ x1, x2, ..., xn]will construct a linear epression to be used 
+      [ x1, x2, ..., xn] will construct a linear epression to be used 
       as a constraint or variable
+
+Comments, bug reports, patches and suggestions are welcome.
+pulp-or-discuss@googlegroups.com
+
+References:
+[1] http://www.gnu.org/software/glpk/glpk.html
+[2] http://www.coin-or.org/
+[3] http://www.cplex.com/
+[4] http://www.gurobi.com/
 """
 
 import types
@@ -458,35 +473,32 @@ class LpVariable(LpElement):
 
 class LpAffineExpression(odict.OrderedDict):
     """
-    A linear combination of LP variables
-
-    An LpAffine Expression can be initalised with the following:
-    
+    A linear combination of :class:`LpVariables<LpVariable>`.
+    Can be initialised with the following:
+  
     #.   e = None: an empty Expression
-    #.   e = dict: gives an expression with the values being the coefficents of the keys
-    #.   e = list: equivalent to dict.items()
-    #.   e = LpElement: an expression of length 1 with the coefficent 1
-    #.   e = other: the constant is intialised as e
+    #.   e = dict: gives an expression with the values being the coefficients of the keys (order of terms is undetermined)
+    #.   e = list or generator of 2-tuples: equivalent to dict.items()
+    #.   e = LpElement: an expression of length 1 with the coefficient 1
+    #.   e = other: the constant is initialised as e
 
-    Examples::
+    Examples:
 
-    >>> f=LpAffineExpression(LpElement('x'))
-    >>> f
-    1*x + 0
-    >>> d = LpAffineExpression(dict(x_0=1, x_1=-3, x_2=4))
-    >>> d
-    1*x_0 + -3*x_1 + 4*x_2 + 0
-
-    >>> x_name = ['x_0', 'x_1', 'x_2']
-    >>> x = [LpVariable(x_name[i], lowBound = 0, upBound = 10) for i in range(3) ]
-    >>> c = LpAffineExpression([ (x[0],1), (x[1],-3), (x[2],4)])
-    >>> c
-    1*x_0 + -3*x_1 + 4*x_2 + 0
-    >>> c == d
-    1*x_0 + -3*x_1 + 4*x_2 + -1*x_0 + 3*x_1 + -4*x_2 + 0 = 0
-
-    >>> ( c == d) == LpConstraint(0)
-    <class 'pulp.pulp.LpConstraint'>
+       >>> f=LpAffineExpression(LpElement('x'))
+       >>> f
+       1*x + 0
+       >>> d = LpAffineExpression(dict(x_0=1, x_1=-3, x_2=4))
+       >>> d
+       1*x_0 + -3*x_1 + 4*x_2 + 0
+       >>> x_name = ['x_0', 'x_1', 'x_2']
+       >>> x = [LpVariable(x_name[i], lowBound = 0, upBound = 10) for i in range(3) ]
+       >>> c = LpAffineExpression([ (x[0],1), (x[1],-3), (x[2],4)])
+       >>> c
+       1*x_0 + -3*x_1 + 4*x_2 + 0
+       >>> c == d
+       1*x_0 + -3*x_1 + 4*x_2 + -1*x_0 + 3*x_1 + -4*x_2 + 0 = 0
+       >>> ( c == d) == LpConstraint(0)
+       <class 'pulp.pulp.LpConstraint'>
 
     """
     #to remove illegal characters from the names
@@ -514,7 +526,7 @@ class LpAffineExpression(odict.OrderedDict):
         elif isinstance(e,dict):
             self.constant = constant
             super(LpAffineExpression,self).__init__(e.items())
-        elif isinstance(e, list):
+        elif isinstance(e, list) or isinstance(e, types.GeneratorType):
             self.constant = constant
             super(LpAffineExpression,self).__init__(e)
         elif isinstance(e,LpElement):
@@ -758,6 +770,12 @@ class LpConstraint(LpAffineExpression):
     """An LP constraint"""
     def __init__(self, e = None, sense = LpConstraintEQ,
                   name = None, rhs = None):
+        """
+        :param e: an instance of :class:`LpAffineExpression`
+        :param sense: one of :data:`~pulp.constants.LpConstraintEQ`, :data:`~pulp.constants.LpConstraintGE`, :data:`~pulp.constants.LpConstraintLE` (0, 1, -1 respectively)
+        :param name: identifying string 
+        :param rhs: numerical value of constraint target
+        """
         LpAffineExpression.__init__(self, e, name = name)
         if rhs is not None:
             self.constant = - rhs
@@ -793,7 +811,7 @@ class LpConstraint(LpAffineExpression):
                 sl += ns
         if not self: sl += "0"
         c = -self.constant
-        if c == 0: c = 0 # Supress sign
+        if c == 0: c = 0 # Suppress sign
         ns = " %s %.12g" % (LpConstraintSenses[self.sense], c)
         if len(sl)+len(ns) > LpCplexLPLineSize:
             s += sl + "\n" + ns + "\n"
@@ -803,7 +821,7 @@ class LpConstraint(LpAffineExpression):
 
     def changeRHS(self, RHS):
         """
-        alters the RHS of a constraint so that it can be modefied in a resolve
+        alters the RHS of a constraint so that it can be modified in a resolve
         """
         self.constant = -RHS
         self.modified = True
@@ -932,11 +950,11 @@ class LpFractionConstraint(LpConstraint):
         numerator/denominator {==, >=, <=} RHS        
         numerator/(numerator + complement) {==, >=, <=} RHS
         
-        @param numerator: the top of the fraction
-        @param denominator: as described above
-        @param sense: the sense of the relation of the constraint
-        @param RHS: the target fraction value
-        @param complement: as described above
+        :param numerator: the top of the fraction
+        :param denominator: as described above
+        :param sense: the sense of the relation of the constraint
+        :param RHS: the target fraction value
+        :param complement: as described above
         """
         self.numerator = numerator
         if denominator is None and complement is not None:
@@ -1003,12 +1021,11 @@ class LpProblem(object):
         
         This function creates a new LP Problem  with the specified associated parameters
             
-        Inputs:
-            - name -- The name of the problem used in the output .lp file
-            - sense(optional) -- The LP problem objective: LpMinimize(default) or LpMaximise
-                    
-        Returns:
-            - An LP Problem
+        :param name: name of the problem used in the output .lp file
+        :param sense: of the LP problem objective.  \
+                Either :data:`~pulp.constants.LpMinimize` (default) \
+                or :data:`~pulp.constants.LpMaximize`.
+        :return: An LP Problem
         """
         self.objective = None
         self.constraints = odict.OrderedDict()
@@ -1180,8 +1197,7 @@ class LpProblem(object):
         """
         Sets the input variable as the objective function. Used in Columnwise Modelling
         
-        Inputs:
-            - obj -- the objective function of type LpConstraintVar
+        :param obj: the objective function of type :class:`LpConstraintVar`
             
         Side Effects:
             - The objective function is set
@@ -1348,7 +1364,7 @@ class LpProblem(object):
         f.write("ENDATA\n")
         f.close()
         self.restoreObjective(wasNone, dummyVar)
-        # returns the variables, in writting order
+        # returns the variables, in writing order
         if rename == 0:
             return vs
         else:
@@ -1361,8 +1377,7 @@ class LpProblem(object):
         This function writes the specifications (objective function,
         constraints, variables) of the defined Lp problem to a file.
         
-        Inputs:
-            - filename -- the name of the file to be created.          
+        :param filename:  the name of the file to be created.          
                 
         Side Effects:
             - The file is created.
@@ -1468,10 +1483,9 @@ class LpProblem(object):
         Solve the given Lp problem.
         
         This function changes the problem to make it suitable for solving
-        then calls the solver.actualSolve() method to find the solution
+        then calls the solver.actualSolve method to find the solution
         
-        Inputs:
-            - solver -- Optional: the specific solver to be used, defaults to the 
+        :param solver:  Optional: the specific solver to be used, defaults to the 
               default solver.
                 
         Side Effects:
@@ -1493,16 +1507,16 @@ class LpProblem(object):
     def sequentialSolve(self, objectives, absoluteTols = None,
                         relativeTols = None, solver = None, debug = False):
         """
-        Solve the given Lp problem with several objective fuctions.
+        Solve the given Lp problem with several objective functions.
         
         This function sequentially changes the objective of the problem
         and then adds the objective function as a constraint
         
-        @param objectives: the list of objectives to be used to solve the problem
-        @param absoluteTols: the list of absolute tolerances to be applied to 
+        :param objectives: the list of objectives to be used to solve the problem
+        :param absoluteTols: the list of absolute tolerances to be applied to 
            the constraints should be +ve for a minimise objective
-        @param relativeTols: the list of relative tolerances applied to the constraints
-        @param solver: the specific solver to be used, defaults to the default solver.
+        :param relativeTols: the list of relative tolerances applied to the constraints
+        :param solver: the specific solver to be used, defaults to the default solver.
         
         """
         #TODO Add a penalty variable to make problems elastic
@@ -1552,15 +1566,16 @@ class LpProblem(object):
     
 class FixedElasticSubProblem(LpProblem):
     """
-    Contains the subproblem generated by converting an fixed constraint 
-    \sum_{i}a_i x_i = b
-    into an elastic constraint
-    @param constraint: The LpConstraint that the elastic constraint is based on
-    @param penalty: penalty applied for violation (+ve or -ve) of the constraints
-    @param proportionFreeBound: the proportional bound (+ve and -ve) on 
+    Contains the subproblem generated by converting a fixed constraint 
+    :math:`\sum_{i}a_i x_i = b` into an elastic constraint.
+
+    :param constraint: The LpConstraint that the elastic constraint is based on
+    :param penalty: penalty applied for violation (+ve or -ve) of the constraints
+    :param proportionFreeBound: 
+        the proportional bound (+ve and -ve) on 
         constraint violation that is free from penalty
-    @param proportionFreeBoundList: the proportional bound on 
-        constraint violation that is free from penalty, expressed as a list 
+    :param proportionFreeBoundList: the proportional bound on \
+        constraint violation that is free from penalty, expressed as a list\
         where [-ve, +ve]
     """
     def __init__(self, constraint, penalty = None,
@@ -1584,6 +1599,7 @@ class FixedElasticSubProblem(LpProblem):
             # variable
             freeVar = LpVariable(name, lowBound = -upbound, upBound = +lowbound)
             constraint.addInPlace(freeVar)
+
             self.freeVar = freeVar
         if penalty:
             #add two penalty variables  
@@ -1623,14 +1639,15 @@ class FixedElasticSubProblem(LpProblem):
 
     def findDifferenceFromRHS(self):
         """
-        This is the amount the actual value varies from the RHS
+        The amount the actual value varies from the RHS (sense: LHS - RHS)
         """
         return self.findLHSValue() - self.RHS
 
+
     def findLHSValue(self):
         """
-        for elastic constraints finds the LHS value of the constraint without 
-        the free variable and or penalty variable assumes the constant is on the
+        finds the LHS value of the constraint (without 
+        the free variable and/or penalty variable) - assumes the constant is on the
         rhs
         """
         upVar = self._findValue("upVar")
@@ -1639,6 +1656,7 @@ class FixedElasticSubProblem(LpProblem):
         return self.constraint.value() - self.constant - \
                 upVar - lowVar - freeVar
     
+
     def alterName(self, name):
         """
         Alters the name of anonymous parts of the problem
@@ -1655,11 +1673,11 @@ class FixedElasticSubProblem(LpProblem):
 
 class FractionElasticSubProblem(FixedElasticSubProblem):
     """
-    Contains the subproblem generated by converting an Fraction constraint 
+    Contains the subproblem generated by converting a Fraction constraint 
     numerator/(numerator+complement) = b
     into an elastic constraint
     
-    :param constraint: The LpFractionConstraint that the elastic constraint is 
+    :param constraint: The LpFractionConstraint that the elastic constraint is\
         based on
     :param penalty: penalty applied for violation (+ve or -ve) of the constraints
     :param proportionFreeBound: the proportional bound (+ve and -ve) on 
@@ -1776,7 +1794,7 @@ def value(x):
 
 def valueOrDefault(x):
     """Returns the value of the variable/expression x, or x if it is a number
-    Variable wihout value (None) are affected a possible value (within their 
+    Variable without value (None) are affected a possible value (within their 
     bounds)."""
     if isNumber(x): return x
     else: return x.valueOrDefault()
@@ -1873,7 +1891,7 @@ def __permutation(orgset, k):
 
 def allpermutations(orgset,k):
     """
-    returns all purmutations of orgset with up to k items
+    returns all permutations of orgset with up to k items
     
     :param orgset: the list to be iterated
     :param k: the maxcardinality of the subsets
@@ -1906,7 +1924,7 @@ def allpermutations(orgset,k):
 
 def allcombinations(orgset,k):
     """
-    returns all purmutations of orgset with up to k items
+    returns all permutations of orgset with up to k items
     
     :param orgset: the list to be iterated
     :param k: the maxcardinality of the subsets
@@ -2009,6 +2027,8 @@ def pulpTestCheck(prob, solver, okstatus, sol = {},
         prob.writeLP("debug.lp")
         prob.writeMPS("debug.mps")
         print "Failure: status ==", status, "not in", okstatus
+        print "Failure: status ==", LpStatus[status], "not in", \
+                                [LpStatus[s] for s in okstatus]
         raise PulpError, "Tests failed for solver %s"%solver
     if sol:
         for v,x in sol.iteritems():
@@ -2085,14 +2105,18 @@ def pulpTest012(solver):
     prob += -y+z == 7, "c3"
     prob += w >= 0, "c4"
     print "\t Testing unbounded continuous LP solution"
-    if solver.__class__ in [COINMP_DLL, GUROBI, CPLEX_CMD,]:
-            # COINMP_DLL Does not report unbounded problems, correctly
-             pulpTestCheck(prob, solver, [LpStatusInfeasible])
+    if solver.__class__ in [GUROBI, CPLEX_CMD,]:
+        # These solvers report infeasible or unbounded
+        pulpTestCheck(prob, solver, [LpStatusInfeasible])
+    elif solver.__class__ in [COINMP_DLL,]:
+        # COINMP_DLL is just plain wrong
+        print '\t\t Error in CoinMP it reports Optimal'
+        pulpTestCheck(prob, solver, [LpStatusOptimal])
     elif solver.__class__ is GLPK_CMD:
         # GLPK_CMD Does not report unbounded problems, correctly
          pulpTestCheck(prob, solver, [LpStatusUndefined])
     elif solver.__class__ in [CPLEX_DLL,]:
-        # GLPK_CMD Does not report unbounded problems, correctly
+        # CPLEX_DLL Does not report unbounded problems, correctly
          pulpTestCheck(prob, solver, [LpStatusNotSolved])
     else:
         pulpTestCheck(prob, solver, [LpStatusUnbounded])
@@ -2147,15 +2171,14 @@ def pulpTest050(solver):
     prob += x+y <= 5.2, "c1"
     prob += x+z >= 10.3, "c2"
     prob += -y+z == 17.5, "c3"
+    print "\t Testing an infeasible problem"
     if solver.__class__ is GLPK_CMD:
-        # GLPK_CMD return codes are not enough informative
+        # GLPK_CMD return codes are not informative enough
         pulpTestCheck(prob, solver, [LpStatusUndefined])
     elif solver.__class__ is CPLEX_DLL:
         # CPLEX_DLL Does not solve the problem
-        print "\t Testing an infeasible problem"
         pulpTestCheck(prob, solver, [LpStatusNotSolved])
     else:
-        print "\t Testing an infeasible problem"
         pulpTestCheck(prob, solver, [LpStatusInfeasible])
 
 def pulpTest060(solver):
@@ -2167,17 +2190,16 @@ def pulpTest060(solver):
     prob += x+y <= 5.2, "c1"
     prob += x+z >= 10.3, "c2"
     prob += -y+z == 7.4, "c3"
+    print "\t Testing an integer infeasible problem"
     if solver.__class__ in [GLPK_CMD, COIN_CMD]:
         # GLPK_CMD returns InfeasibleOrUnbounded
-        print "\t Testing an integer infeasible problem"
         pulpTestCheck(prob, solver, [LpStatusInfeasible, LpStatusUndefined])
     elif solver.__class__ in [COINMP_DLL]:
         #Currently there is an error in COINMP for problems where
-        #presolve elimiates too many variables
-        print "\t Testing an integer infeasible problem (Error to be fixed)"
+        #presolve eliminates too many variables
+        print "\t\t Error in CoinMP to be fixed, reports Optimal"
         pulpTestCheck(prob, solver, [LpStatusOptimal])
     else:
-        print "\t Testing an integer infeasible problem"
         pulpTestCheck(prob, solver, [LpStatusInfeasible])
 
 def pulpTest070(solver):
@@ -2396,7 +2418,7 @@ def pulpTest123(solver):
         pulpTestCheck(prob, solver, [LpStatusUnbounded])
 
 
-def pulpTestSolver(solver):
+def pulpTestSolver(solver, msg = 0):
     tests = [pulpTest010, pulpTest011, pulpTest012, 
             pulpTest020, 
             pulpTest030, 
@@ -2411,7 +2433,7 @@ def pulpTestSolver(solver):
             pulpTest120, pulpTest121, pulpTest122, pulpTest123
             ]
     for t in tests:
-        t(solver(msg=0))
+        t(solver(msg=msg))
 
 def pulpTestAll():
     solvers = [CPLEX_DLL, 
