@@ -50,20 +50,37 @@ class PulpSolverError(Exception):
     pass
 
 #import configuration information
-def initialize(file = None):
+def initialize(filename):
     """ reads the configuration file to initialise the module"""
-    if not file:
-        file = __file__
-    config = ConfigParser.SafeConfigParser()
-    config.read(file)
-    cplex_dll_path = config.get("locations", "CplexPath")
-    coinMP_path = config.get("locations", "CoinMPPath").split()
-    gurobi_path = config.get("locations", "GurobiPath")
+    here = os.path.dirname(filename)
+    config = ConfigParser.SafeConfigParser({'here':here})
+    config.read(filename)
+    print here
+    try:
+        cplex_dll_path = config.get("locations", "CplexPath")
+    except ConfigParser.NoOptionError:
+        cplex_dll_path = 'libcplex110.so'
+    try:
+        coinMP_path = config.get("locations", "CoinMPPath").split()
+    except ConfigParser.NoOptionError:
+        coinMP_path = 'libCoinMP.so'
+    try:
+        gurobi_path = config.get("locations", "GurobiPath")
+    except ConfigParser.NoOptionError:
+        gurobi_path = '/opt/gurobi201/linux32/lib/python2.5'
+    try:
+        cbc_path = config.get("locations", "CbcPath")
+    except ConfigParser.NoOptionError:
+        cbc_path = 'cbc'
+    try:
+        glpk_path = config.get("locations", "GlpkPath")
+    except ConfigParser.NoOptionError:
+        glpk_path = 'glpsol'
     for i,path in enumerate(coinMP_path):
         if not os.path.dirname(path):
             #if no pathname is supplied assume the file is in the same directory
             coinMP_path[i] = os.path.join(os.path.dirname(config_filename),path) 
-    return cplex_dll_path, coinMP_path, gurobi_path
+    return cplex_dll_path, coinMP_path, gurobi_path, cbc_path, glpk_path
 
 #pick up the correct config file depending on operating system
 PULPCFGFILE = "pulp.cfg"
@@ -73,13 +90,17 @@ else:
     PULPCFGFILE += ".linux"
     
 if __name__ != '__main__':
-    config_filename = os.path.join(os.path.dirname(__file__),
+    DIRNAME = os.path.dirname(__file__)
+    config_filename = os.path.join(DIRNAME,
                                    PULPCFGFILE)
 else: #run as a script
     from pulp import __file__ as fname
-    config_filename = os.path.join(os.path.dirname(fname),
+    DIRNAME = os.path.dirname(fname)
+    config_filename = os.path.join(DIRNAME,
                                    PULPCFGFILE)
-cplex_dll_path, coinMP_path, gurobi_path = initialize(config_filename)
+cplex_dll_path, coinMP_path, gurobi_path, cbc_path, glpk_path = \
+        initialize(config_filename)
+
 
 # See later for LpSolverDefault definition
 class LpSolver:
@@ -275,7 +296,7 @@ class LpSolver_CMD(LpSolver):
 class GLPK_CMD(LpSolver_CMD):
     """The GLPK LP solver"""
     def defaultPath(self):
-        return self.executableExtension("glpsol")
+        return self.executableExtension(glpk_path)
 
     def available(self):
         """True if the solver is available"""
@@ -909,21 +930,28 @@ class COIN_CMD(LpSolver_CMD):
     """The COIN CLP/CBC LP solver
     now only uses cbc
     """
+
     def defaultPath(self):
-        return self.executableExtension("cbc")
+        return self.executableExtension(cbc_path)
 
     def __init__(self, path = None, keepFiles = 0, mip = 1,
-            msg = 1, cuts = 1, presolve = 1, dual = 1, strong = 5, options = []):
-        """Here, path is a tuple containing the path to clp and cbc"""
+            msg = 0, cuts = None, presolve = None, dual = None,
+            strong = None, options = [],
+            fracGap = None, maxSeconds = None, threads = None):
         LpSolver_CMD.__init__(self, path, keepFiles, mip, msg, options)
         self.cuts = cuts
         self.presolve = presolve
         self.dual = dual
         self.strong = strong
+        self.fracGap = fracGap
+        self.maxSeconds = maxSeconds
+        self.threads = threads
+        #TODO hope this gets fixed in cbc as it does not like the c:\ in windows paths
+        if os.name == 'nt':
+            self.tmpDir = ''
 
     def copy(self):
         """Make a copy of self"""
-        
         aCopy = LpSolver_CMD.copy(self)
         aCopy.cuts = self.cuts
         aCopy.presolve = self.presolve
@@ -934,8 +962,6 @@ class COIN_CMD(LpSolver_CMD):
     def actualSolve(self, lp):
         """Solve a well formulated lp problem"""
         return self.solve_CBC(lp)
-        #if lp.isMIP() and self.mip: return self.solve_CBC(lp)
-        #else: return self.solve_CLP(lp)
 
     def available(self):
         """True if the solver is available"""
@@ -944,7 +970,8 @@ class COIN_CMD(LpSolver_CMD):
     def solve_CBC(self, lp):
         """Solve a MIP problem using CBC"""
         if not self.executable(self.path):
-            raise PulpSolverError, "PuLP: cannot execute "+self.path
+            raise PulpSolverError, "Pulp: cannot execute %s cwd: %s"%(self.path,
+                                   os.getcwd())
         if not self.keepFiles:
             pid = os.getpid()
             tmpLp = os.path.join(self.tmpDir, "%d-pulp.lp" % pid)
@@ -953,41 +980,55 @@ class COIN_CMD(LpSolver_CMD):
             tmpLp = lp.name+"-pulp.lp"
             tmpSol = lp.name+"-pulp.sol"
         lp.writeLP(tmpLp)
-        if not self.msg and os.name == 'posix':
-            cbc = os.popen(self.path+" - > /dev/null 2> /dev/null","w")
-        else:
-            cbc = os.popen(self.path+" -","w")
-        cbc.write("import "+tmpLp+"\n")
+        cmds = ' '+tmpLp+" "
+        if self.threads:
+            cmds += "threads %s "%self.threads
+        if self.fracGap is not None:
+            cmds += "ratio %s "%self.fracGap
+        if self.maxSeconds is not None:
+            cmds += "sec %s "%self.maxSeconds
         if self.presolve:
-            cbc.write("presolve on\n")
-        cbc.write("strong %d\n" % self.strong)
+            cmds += "presolve on "
+        if self.strong:
+            cmds += "strong %d " % self.strong
         if self.cuts:
-            cbc.write("gomory on\n")
-            #cbc.write("oddhole on\n")
-            cbc.write("knapsack on\n")
-            cbc.write("probing on\n")
+            cmds += "gomory on "
+            #cbc.write("oddhole on "
+            cmds += "knapsack on "
+            cmds += "probing on "
         for option in self.options:
-            cbc.write(option+"\n")
+            cmds += option+" "
         if self.mip:
-            cbc.write("branch\n")
+            cmds += "branch "
         else:
-            cbc.write("initialSolve\n")
-        cbc.write("solution "+tmpSol+"\n")
-        cbc.write("quit\n")
-        if cbc.close() != None:
-            raise PulpSolverError, "PuLP: Error while trying to execute "+self.path
+            cmds += "initialSolve "
+        cmds += "solution "+tmpSol+" "
+        if self.msg:
+            pipe = None
+        else:
+            pipe = open(os.devnull, 'w')
+        logging.debug(self.path + cmds)
+        cbc = subprocess.Popen((self.path + cmds).split(), stdout = pipe,
+                             stderr = pipe)
+        if cbc.wait() != 0:
+            raise PulpSolverError, "Pulp: Error while trying to execute " +  \
+                                    self.path
         if not os.path.exists(tmpSol):
-            raise PulpSolverError, "PuLP: Error while executing "+self.path
+            raise PulpSolverError, "Pulp: Error while executing "+self.path
         lp.status, values = self.readsol_CBC(tmpSol, lp, lp.variables())
         lp.assignVarsVals(values)
         if not self.keepFiles:
-            try: os.remove(tmpLp)
-            except: pass
-            try: os.remove(tmpSol)
-            except: pass
+            try:
+                os.remove(tmpLp)
+            except:
+                pass
+            try:
+                os.remove(tmpSol)
+            except:
+                pass
         return lp.status
 
-    def readsol_CBC(self,filename, lp, vs):
+    def readsol_CBC(self, filename, lp, vs):
         """Read a CBC solution file"""
         values = {}
         for v in vs:
@@ -1000,44 +1041,48 @@ class COIN_CMD(LpSolver_CMD):
         statusstr = f.readline().split()[0]
         status = cbcStatus.get(statusstr, LpStatusUndefined)
         for l in f:
-            if len(l)<=2: break
+            if len(l)<=2:
+                break
             l = l.split()
             vn = l[1]
             if vn in values:
                 values[vn] = float(l[2])
         return status, values
 
-try:
-    import ctypes
-    #CoinMP=ctypes.cdll.LoadLibrary(coinMP_path)
-    #linux hack to get working
-    for libpath in coinMP_path[:-1]:
-        ctypes.CDLL(libpath, mode = ctypes.RTLD_GLOBAL)
-    CoinMP  = ctypes.CDLL(coinMP_path[-1], mode = ctypes.RTLD_GLOBAL)
-except (ImportError,OSError):
+COIN = COIN_CMD
 
-    class COINMP_DLL(LpSolver):
-        """The COIN_MP LP MIP solver (via a DLL Windows only)"""
-        def available(self):
+
+class COINMP_DLL(LpSolver):
+    """The COIN_MP LP MIP solver (via a DLL or linux so)"""
+    try:
+        import ctypes
+        if os.name == 'nt':
+            lib = ctypes.windll.LoadLibrary(coinMP_path[-1])
+        else:
+            #linux hack to get working
+            for libpath in coinMP_path[:-1]:
+                #RTLD_LAZY = 0x00001
+                mode = ctypes.RTLD_GLOBAL
+                ctypes.CDLL(libpath, mode = mode)
+            lib = ctypes.CDLL(coinMP_path[-1], mode = mode)
+    except (ValueError):#(ImportError,OSError):
+        @classmethod
+        def available(cls):
             """True if the solver is available"""
             return False
         def actualSolve(self, lp):
             """Solve a well formulated lp problem"""
             raise PulpSolverError, "COINMP_DLL: Not Available"
-
-    COIN = COIN_CMD
-else:    
-    class COINMP_DLL(LpSolver):
-        """The COIN_MP LP/MIP solver (via a DLL windows only)"""
+    else:    
         COIN_INT_LOGLEVEL = 7
         COIN_REAL_MAXSECONDS = 16
         COIN_REAL_MIPMAXSEC = 19
         COIN_REAL_MIPFRACGAP = 34
-        CoinMP.CoinGetInfinity.restype = ctypes.c_double
-        CoinMP.CoinGetVersionStr.restype = ctypes.c_char_p
-        CoinMP.CoinGetSolutionText.restype=ctypes.c_char_p
-        CoinMP.CoinGetObjectValue.restype=ctypes.c_double
-        CoinMP.CoinGetMipBestBound.restype=ctypes.c_double
+        lib.CoinGetInfinity.restype = ctypes.c_double
+        lib.CoinGetVersionStr.restype = ctypes.c_char_p
+        lib.CoinGetSolutionText.restype=ctypes.c_char_p
+        lib.CoinGetObjectValue.restype=ctypes.c_double
+        lib.CoinGetMipBestBound.restype=ctypes.c_double
 
         def __init__(self, mip = 1, msg = 1, cuts = 1, presolve = 1, dual = 1,
             crash = 0, scale = 1, rounding = 1, integerPresolve = 1, strong = 5,
@@ -1073,7 +1118,8 @@ else:
             aCopy.strong = self.strong
             return aCopy
 
-        def available(self):
+        @classmethod
+        def available(cls):
             """True if the solver is available"""
             return True
 
@@ -1085,30 +1131,30 @@ else:
             >>> COINMP_DLL().getSolverVersion() # doctest: +ELLIPSIS
             '...'
             """
-            return CoinMP.CoinGetVersionStr()
+            return self.lib.CoinGetVersionStr()
             
         def actualSolve(self, lp):
             """Solve a well formulated lp problem"""
             #TODO alter so that msg parameter is handled correctly
             self.debug = 0
             #initialise solver
-            CoinMP.CoinInitSolver("")
+            self.lib.CoinInitSolver("")
             #create problem
-            hProb = CoinMP.CoinCreateProblem(lp.name);
+            self.hProb = hProb = self.lib.CoinCreateProblem(lp.name);
             #set problem options
             if self.maxSeconds:
                 if self.mip:
-                    CoinMP.CoinSetRealOption(hProb, self.COIN_REAL_MIPMAXSEC,
+                    self.lib.CoinSetRealOption(hProb, self.COIN_REAL_MIPMAXSEC,
                                           ctypes.c_double(self.maxSeconds))
                 else:
-                    CoinMP.CoinSetRealOption(hProb, self.COIN_REAL_MAXSECONDS,
+                    self.lib.CoinSetRealOption(hProb, self.COIN_REAL_MAXSECONDS,
                                           ctypes.c_double(self.maxSeconds))
             if self.fracGap:
                #Hopefully this is the bound gap tolerance
-               CoinMP.CoinSetRealOption(hProb, self.COIN_REAL_MIPFRACGAP,
+               self.lib.CoinSetRealOption(hProb, self.COIN_REAL_MIPFRACGAP,
                                           ctypes.c_double(self.fracGap))
             #CoinGetInfinity is needed for varibles with no bounds
-            coinDblMax = CoinMP.CoinGetInfinity()
+            coinDblMax = self.lib.CoinGetInfinity()
             if self.debug: print "Before getCoinMPArrays"
             (numVars, numRows, numels, rangeCount,
                 objectSense, objectCoeffs, objectConst, 
@@ -1116,18 +1162,15 @@ else:
                 lenBase, indBase, 
                 elemBase, lowerBounds, upperBounds, initValues, colNames, 
                 rowNames, columnType, n2v, n2c) = self.getCplexStyleArrays(lp)
-            CoinMP.CoinLoadProblem(hProb, numVars, numRows,
-                                   numels, rangeCount,
-                                   objectSense, objectCoeffs,
-                                   objectConst,
-                                   rhsValues, rangeValues,
-                                   rowType, startsBase,
-                                   lenBase, indBase,
-                                   elemBase, lowerBounds,
-                                   upperBounds, initValues,
-                                   colNames, rowNames )
+            self.lib.CoinLoadProblem(hProb,
+                                   numVars, numRows, numels, rangeCount,
+                                   objectSense, objectConst, objectCoeffs,
+                                   lowerBounds, upperBounds, rowType,
+                                   rhsValues, rangeValues, startsBase,
+                                   lenBase, indBase, elemBase,
+                                   colNames, rowNames, "Objective")
             if lp.isMIP() and self.mip:
-                CoinMP.CoinLoadInteger(hProb,columnType)
+                self.lib.CoinLoadInteger(hProb,columnType)
             if self.msg == 0:
                 #close stdout to get rid of messages
                 tempfile = open(mktemp(),'w')
@@ -1136,7 +1179,7 @@ else:
                 if os.dup(tempfile.fileno()) != 1:
                     raise PulpSolverError, "couldn't redirect stdout - dup() error"
             self.coinTime = -clock()
-            CoinMP.CoinOptimizeProblem(hProb, 0);
+            self.lib.CoinOptimizeProblem(hProb, 0);
             self.coinTime += clock()
 
             if self.msg == 0:
@@ -1153,9 +1196,9 @@ else:
                             5:LpStatusNotSolved,
                             -1:LpStatusUndefined
                             }
-            solutionStatus = CoinMP.CoinGetSolutionStatus(hProb)
-            solutionText = CoinMP.CoinGetSolutionText(hProb,solutionStatus)
-            objectValue =  CoinMP.CoinGetObjectValue(hProb)
+            solutionStatus = self.lib.CoinGetSolutionStatus(hProb)
+            solutionText = self.lib.CoinGetSolutionText(hProb,solutionStatus)
+            objectValue =  self.lib.CoinGetObjectValue(hProb)
 
             #get the solution values
             NumVarDoubleArray = ctypes.c_double * numVars
@@ -1164,7 +1207,7 @@ else:
             cReducedCost = NumVarDoubleArray()
             cSlackValues = NumRowsDoubleArray()
             cShadowPrices = NumRowsDoubleArray()
-            CoinMP.CoinGetSolutionValues(hProb, ctypes.byref(cActivity),
+            self.lib.CoinGetSolutionValues(hProb, ctypes.byref(cActivity),
                                          ctypes.byref(cReducedCost),
                                          ctypes.byref(cSlackValues),
                                          ctypes.byref(cShadowPrices))
@@ -1174,7 +1217,7 @@ else:
             constraintpivalues = {}
             constraintslackvalues = {}
             if lp.isMIP() and self.mip:
-                lp.bestBound = CoinMP.CoinGetMipBestBound(hProb)
+                lp.bestBound = self.lib.CoinGetMipBestBound(hProb)
             for i in range(numVars):
                 variablevalues[self.n2v[i].name] = cActivity[i]
                 variabledjvalues[self.n2v[i].name] = cReducedCost[i]
@@ -1183,14 +1226,16 @@ else:
             #put pi and slack variables against the constraints
             for i in range(numRows):
                 constraintpivalues[self.n2c[i]] = cShadowPrices[i]
-                constraintslackvalues[self.n2c[i]] = rhsValues[i] - cSlackValues[i]                
+                constraintslackvalues[self.n2c[i]] = \
+                    rhsValues[i] - cSlackValues[i]                
             lp.assignConsPi(constraintpivalues)
             lp.assignConsSlack(constraintslackvalues)
                        
-            CoinMP.CoinFreeSolver()
-            lp.status = CoinLpStatus[CoinMP.CoinGetSolutionStatus(hProb)]
+            self.lib.CoinFreeSolver()
+            lp.status = CoinLpStatus[self.lib.CoinGetSolutionStatus(hProb)]
             return lp.status        
-    
+
+if COINMP_DLL.available():
     COIN = COINMP_DLL
 
 try:
